﻿//Copyright (c) 2009, Mark S. Burrell
//All rights reserved.

//Redistribution and use in source and binary forms, with or without modification, are 
//permitted provided that the following conditions are met:

//    * Redistributions of source code must retain the above copyright notice, this list 
//of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this 
//list of conditions and the following disclaimer in the documentation and/or other 
//materials provided with the distribution.

//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
//EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
//OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
//SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
//OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
//SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using Sedici.Hardware;


namespace AssemblerGUI
{
    public partial class AssemblerGUI : Form
    {
        SediciAssembler assembler;

        public AssemblerGUI()
        {
            InitializeComponent();

            assembler = new SediciAssembler();

            labelVersion.Text = globalConstants.COPYRIGHT + " " + globalConstants.VERSION;

            string firmware = "firmware.frm";

            // load the firmware
            supportMethods.updateFileLocation(ref firmware, globalConstants.envName);
            assembler.loadFirmware(firmware);

        }


        private void buttonFirmware_Click(object sender, EventArgs e)
        {

            // read the file into a data structure
            OpenFileDialog dlgOpen = new OpenFileDialog();

            dlgOpen.Filter = "Firmware Files (*.frm)|*.frm|All Files (*.*)|*.*";
            dlgOpen.Title = "Open Firmware File ...";

            if (dlgOpen.ShowDialog() == DialogResult.OK)
            {
                assembler.loadFirmware(dlgOpen.FileName);
            }

        }

        private void buttonOpen_Click(object sender, EventArgs e)
        {
            // read the file into a data structure
            OpenFileDialog dlgOpen = new OpenFileDialog();

            dlgOpen.Filter = "Assembler Source Files (*.src)|*.src|All Files (*.*)|*.*";
            dlgOpen.Title = "Open Assembler Source File ...";

            if (dlgOpen.ShowDialog() == DialogResult.OK)
            {

                textDisplay.Text = "";

                assembler.loadAssemblyCode(dlgOpen.FileName);

            }
        }

        private void buttonTokenise_Click(object sender, EventArgs e)
        {

            assembler.initialTokenisation();

        }

        private void buttonParse_Click(object sender, EventArgs e)
        {
            assembler.initialCoding();

        }

        private void buttonShowTokens_Click(object sender, EventArgs e)
        {
            textDisplay.Text = assembler.getTokenSet();
        }

        private void buttonIntermediate_Click(object sender, EventArgs e)
        {

            textDisplay.Text = assembler.getIntermediateMachineCode();

        }

        private void buttonPrettyPrint_Click(object sender, EventArgs e)
        {
            textDisplay.Text = assembler.prettyPrint();
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            SaveFileDialog dlgSave = new SaveFileDialog();
            dlgSave.Filter = "Assembler File|*.asm";
            dlgSave.Title = "Save an Assembler File ...";
            dlgSave.ShowDialog();

            if (dlgSave.FileName != "")
            {
                assembler.performSave(dlgSave.FileName);
            }

        }

        private void buttonDSW_Click(object sender, EventArgs e)
        {
            // get rid of DSWdata
            assembler.processDSWReferences();
        }

        private void buttonSymbols_Click(object sender, EventArgs e)
        {

            // now we need to update all SymbolTable values
            assembler.processSetSymbolReferences();

            assembler.processSymbolReferences();

            assembler.processDisplacementSymbolReferences();

        }
        
        private void buttonViewCode_Click(object sender, EventArgs e)
        {
            textDisplay.Text = assembler.getIntermediateMachineCode();
            textDisplay.Text = textDisplay.Text + Environment.NewLine;
            textDisplay.Text = textDisplay.Text + assembler.getSymbolTable();

        }

        private void buttonAssemble_Click(object sender, EventArgs e)
        {
            // initial tokenisation
            textDisplay.Text = assembler.assemble();

        }

        private void buttonMachineCode_Click(object sender, EventArgs e)
        {
            textDisplay.Text = assembler.prettyPrintMachineCode();
        }

        private void buttonMachineCodeChange(object sender, EventArgs e)
        {
            if (checkBoxMachineCode.Checked)
            {
                assembler.setOutputToMachineCode(true);

            }
            else
            {
                assembler.setOutputToMachineCode(false);
            }
        }


    }
}
