﻿//Copyright (c) 2009, Mark S. Burrell
//All rights reserved.

//Redistribution and use in source and binary forms, with or without modification, are 
//permitted provided that the following conditions are met:

//    * Redistributions of source code must retain the above copyright notice, this list 
//of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this 
//list of conditions and the following disclaimer in the documentation and/or other 
//materials provided with the distribution.

//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
//EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
//OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
//SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
//OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
//SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Sedici.Hardware;

using CMMCompiler;

namespace CMMCompiler
{
    class CMMCompiler
    {
        public const string Name = "Sedici C-- Cross Compiler";

        static void Main(string[] args)
        {

            // variables for argument strings, plus a string separator
            string[] rawarguments = { };
            string[] arguments = { };
            string[] separator = { " ", "\t" };
            string[] equals = { "=" };

            string compilerfile = "";

            string message = "";


            // check if we have a file as the first param
            if (args.Length > 0)
            {
                args[0] = convertSynonyms(args[0]);
                switch (args[0])
                {
                    case "/help":
                        showHelp();
                        break;
                    default:
                        compilerfile = args[0];
                        break;
                }

            }
            else
            {
                // no params so show help by default
                showHelp();
            }

            // now look for extra params
            if (args.Length > 1)
            {
                //Check parameters
                for (int i = 1; i < args.Length; i++)
                {
                    // if the argument includes a '=' then separate it out first
                    arguments = args[i].Split(equals, StringSplitOptions.None);
                    arguments[0] = convertSynonyms(arguments[0]);

                    switch (arguments[0])
                    {
                        default:
                            message = "Unknown parameter [" + args[i] + "]";
                            Console.Error.WriteLine(message);
                            break;
                    }
                }
            }
            else
            {
                // no further params
            }



            if (compilerfile == "") {
                //Console.WriteLine("No file to compile");

            }
            else {

                if (File.Exists(compilerfile))
                {
                    Scanner scanner = new Scanner(compilerfile);

                    Parser parser = new Parser(scanner);
                    //parser.tab = new SymbolTable(parser);
                    //parser.gen = new CodeGenerator();
                    parser.Parse();
                    if (parser.errors.count == 0)
                    {
                        Console.Error.WriteLine("Compiled with no errors");

                        string text = parser.gen.getOutput();
                        Console.WriteLine(text);
                    }
                    else if (parser.errors.count == 1)
                    {
                        Console.Error.WriteLine("Compilation failed with 1 error");
                    }
                    else
                    {
                        Console.Error.WriteLine("Compilation failed with " + parser.errors.count + " errors");
                    }
                }
                else
                {
                    message = "Source file does not exist";
                    Console.Error.WriteLine(message);
                }

            }

            // end main
        }

        static void showHelp()
        {

            Console.WriteLine();
            Console.WriteLine(" NAME");
            Console.WriteLine();
            Console.WriteLine("   " + Name);
            Console.WriteLine();
            Console.WriteLine(" DESCRIPTION");
            Console.WriteLine("   This C-- compiler writes output that can be assembled");
            Console.WriteLine("   by 'sasm' and then executed by a Sedici processor based system.");
            Console.WriteLine();
            Console.WriteLine(" NOTE. THIS IS ALPHA SOFTWARE.");
            Console.WriteLine("       THIS PROGRAM DOES NOT FULLY COMPILE ALL SOURCE CODE YET");
            Console.WriteLine();
            Console.WriteLine(" USAGE");
            Console.WriteLine();
            Console.WriteLine("   scmm [ (srcfile | /h*elp | /? ) ]");
            Console.WriteLine();
            Console.WriteLine("        srcfile : the C-- file to be compiled");
            Console.WriteLine("        /help   : list this help information");
            Console.WriteLine();
            Console.WriteLine("   For example, if you had a file called a.cmm to");
            Console.WriteLine("   compile it you would type:");
            Console.WriteLine();
            Console.WriteLine("       scmm a.cmm");
            Console.WriteLine();
            Console.WriteLine("   To capture the output to a separate file use:");
            Console.WriteLine();
            Console.WriteLine("       scmm a.cmm > a.src");
            Console.WriteLine();
            Console.WriteLine();
            Console.WriteLine("   Running under Mono you simply replace 'sasm' in the");
            Console.WriteLine("   examples above with 'mono scmm.exe'");
            Console.WriteLine();
            Console.WriteLine("   It is assumed that the firmware file loads successfully.");
            Console.WriteLine("   This is free software and comes with no warranty.");
            Console.WriteLine();
            Console.WriteLine(" COPYRIGHT");
            Console.WriteLine();
            Console.WriteLine("   " + globalConstants.COPYRIGHT);
            Console.WriteLine();
            Console.WriteLine(" VERSION");
            Console.WriteLine();
            Console.WriteLine("   " + globalConstants.VERSION);
            Console.WriteLine();

            return;
        }


        static string convertSynonyms(string cmd)
        {
            string returnValue = cmd;
            switch (cmd)
            {
                case "l":
                case "lo":
                case "loa":
                case "load":
                    returnValue = "load";
                    break;
                case "/?":
                case "/h":
                case "/he":
                case "/hel":
                case "/help":
                    returnValue = "/help";
                    break;
                default:
                    break;
            }
            return (returnValue);
        }



    }

}
