﻿//Copyright (c) 2009, Mark S. Burrell
//All rights reserved.

//Redistribution and use in source and binary forms, with or without modification, are 
//permitted provided that the following conditions are met:

//    * Redistributions of source code must retain the above copyright notice, this list 
//of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this 
//list of conditions and the following disclaimer in the documentation and/or other 
//materials provided with the distribution.

//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
//EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
//OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
//SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
//OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
//SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

using Sedici.Hardware;

public enum tokenType { INT, BOOL, STRING }; 

public class CompToken
{
    public tokenType type = 0;
    public string id = "";
    public string stringVal = "";
    public bool boolVal = false;
    public int integerVal = 0;
    public bool constant = false;

}

    public class Generator
    {

        //public string id;

        public ArrayList symbolTable = new ArrayList();

        public int label = 0;

        private string text = "";

        private void Write(string s)
        {
            text = text + s + Environment.NewLine;
        }

        public string getOutput()
        {
            return text;
        }

        public string updateString(string s)
        {
            // Replace leading and end " with '
            if (s.Length > 2)
            {
                s = s.Substring(1, (s.Length - 2));
                s = "\'" + s + "\'";
            }
            else
            {
                s = "\'\'";
            }
            return s;
        }

        public void printBanner()
        {
            Write("* " + CMMCompiler.CMMCompiler.Name);
            Write("* " + globalConstants.COPYRIGHT + " " + globalConstants.VERSION);
            Write("*");
            Write("* This is free software and comes with no warranty");
            Write("*");

            // set LPC and ORG
            Write("\tORG  $0100");
            Write("\tLPC  $0100");

            // set stack pointer
            Write("\tMOVE #$DF,USP");
            Write("\tJMP  main");

            // Error messages
            Write("_Eintegeroverflow DC.B \'Integer overflow!\\n\',0");
            Write("_Edividebyzero    DC.B \'Divide by zero!\\n\',0");
            Write("");
        }

        public void printMain()
        {
            Write("main");
        }

        public void printVariableInitialisation()
        {
            // will need to got through the list of symbols and 
            // initialise those that need initialising

            string trueval = "1";
            string falseval = "0";

            Write("* Initialisations here *" + Environment.NewLine);

            CompToken token;
            for (int i = 0; i < symbolTable.Count; i++)
            {
                token = (CompToken)symbolTable[i];
                //Write("got token id [" + token.id + "]");
                if (token.type == tokenType.INT)
                {
                    // init INT
                    Write("\tMOVE #^" + token.integerVal + ",D0");
                    Write("\tMOVE D0," + token.id);
                }
                else if (token.type == tokenType.BOOL)
                {
                    // init BOOL
                    if (token.boolVal == true)
                    {
                        Write("\tMOVE #^" + trueval + ",D0");
                        Write("\tMOVE D0," + token.id);

                    }
                    else
                    {
                        Write("\tMOVE #^" + falseval + ",D0");
                        Write("\tMOVE D0," + token.id);
                    }

                }
                else
                {
                    // string shouldn't need initialisation
                    Write("huh?");
                }
            }

        }

        public void printFooter()
        {
            Write("\tHALT");

            Write("Lintegeroverflow");
            Write("\tMOVE #_Eintegeroverflow,D0");
            Write("\tJSR  putstring");
            Write("\tHALT");

            Write("Ldividebyzero");
            Write("\tMOVE #_Edividebyzero,D0");
            Write("\tJSR  putstring");
            Write("\tHALT");

            Write("\tUSE  basicio.lib");
            Write("");
        }

        public void printConstBool(string id, string boolval)
        {
            string ret = id + "\t DS.W 1";
            Write(ret);

            CompToken token = new CompToken();

            if (boolval.Equals("true", StringComparison.CurrentCultureIgnoreCase))
            {
                token.boolVal = true;
            }
            token.type = tokenType.BOOL;
            token.constant = true;
            token.id = id;

            symbolTable.Add(token);
        }

        public void printConstInt(string id, string i)
        {
            string ret = id + "\t DS.W 1";
            Write(ret);

            ushort result = 0;
            CompToken token = new CompToken();
            supportMethods.isNumber(i, ref result);
            token.integerVal = result;
            token.type = tokenType.INT;
            token.constant = true;
            token.id = id;

            symbolTable.Add(token);
        }

        public void printConstString(string id, string s)
        {
            s = updateString(s);
            string ret = id + "\t DC.B  " + s + ",0";
            Write(ret);
        }

        public void printVariableBool(string id, string boolval)
        {
            string ret = id + "\t DS.W 1";
            Write(ret);

            CompToken token = new CompToken();
            
            if (boolval.Equals("true", StringComparison.CurrentCultureIgnoreCase)) {
                token.boolVal = true;
            }
            token.type = tokenType.BOOL;
            token.id = id;

            symbolTable.Add(token);

        }

        public void printVariableInt(string id, string i)
        {

            string ret = id + "\t DS.W 1";
            Write(ret);

            ushort result = 0;
            CompToken token = new CompToken();

            i = "^" + i; // maaniplute string to say its in decimal
            
            
            supportMethods.isNumber(i, ref result);

            //Write("---->[" + i + "] converted to [" + result + "]");

            token.integerVal = result;
            token.type = tokenType.INT;
            token.id = id;

            symbolTable.Add(token);
        }

        public void printVariableString(string id, string s)
        {
            s = updateString(s);
            string ret = id + "\t DC.B  " + s + ",0";
            Write(ret);
        }

        public void printAssign(string id, string express)
        {
            string ret = "ASSIGN [" + id + "][" + express + "]";
            Write(ret);
        }

    }
