
using System;



public class Parser {
	public const int _EOF = 0;
	public const int _ident = 1;
	public const int _uint = 2;
	public const int _stringlit = 3;
	public const int maxT = 37;

	const bool T = true;
	const bool x = false;
	const int minErrDist = 2;
	
	public Scanner scanner;
	public Errors  errors;

	public Token t;    // last recognized token
	public Token la;   // lookahead token
	int errDist = minErrDist;

public Generator gen = new Generator();

//--------------------------------------------------------------------



	public Parser(Scanner scanner) {
		this.scanner = scanner;
		errors = new Errors();
	}

	void SynErr (int n) {
		if (errDist >= minErrDist) errors.SynErr(la.line, la.col, n);
		errDist = 0;
	}

	public void SemErr (string msg) {
		if (errDist >= minErrDist) errors.SemErr(t.line, t.col, msg);
		errDist = 0;
	}
	
	void Get () {
		for (;;) {
			t = la;
			la = scanner.Scan();
			if (la.kind <= maxT) { ++errDist; break; }

			la = t;
		}
	}
	
	void Expect (int n) {
		if (la.kind==n) Get(); else { SynErr(n); }
	}
	
	bool StartOf (int s) {
		return set[s, la.kind];
	}
	
	void ExpectWeak (int n, int follow) {
		if (la.kind == n) Get();
		else {
			SynErr(n);
			while (!StartOf(follow)) Get();
		}
	}


	bool WeakSeparator(int n, int syFol, int repFol) {
		int kind = la.kind;
		if (kind == n) {Get(); return true;}
		else if (StartOf(repFol)) {return false;}
		else {
			SynErr(n);
			while (!(set[syFol, kind] || set[repFol, kind] || set[0, kind])) {
				Get();
				kind = la.kind;
			}
			return StartOf(syFol);
		}
	}

	
	void MSB_CMM() {
		gen.printBanner(); 
		Declarations();
		gen.printMain();
		gen.printVariableInitialisation();		 
		
		Compstat();
		gen.printFooter(); 
	}

	void Declarations() {
		while (StartOf(1)) {
			Declaration();
			Expect(4);
		}
	}

	void Compstat() {
		Expect(10);
		Statements();
		Expect(11);
	}

	void Declaration() {
		if (la.kind == 5) {
			ConstantDec();
		} else if (la.kind == 6 || la.kind == 8 || la.kind == 9) {
			VariableDec();
		} else SynErr(38);
	}

	void ConstantDec() {
		string b = ""; 
		string s = "";
		string i = "";
		string id = "";
		                                                                   
		Expect(5);
		if (la.kind == 6) {
			Get();
			Ident(out id);
			Expect(7);
			Boolit(out b);
			gen.printConstBool(id, b); 
		} else if (la.kind == 8) {
			Get();
			Ident(out id);
			Expect(7);
			Stringlit(out s);
			gen.printConstString(id, s); 
		} else if (la.kind == 9) {
			Get();
			Ident(out id);
			Expect(7);
			Intlit(out i);
			gen.printConstInt(id, i); 
		} else SynErr(39);
	}

	void VariableDec() {
		string b = ""; 
		string s = "";
		string i = "";
		string id = "";
		                                                                   
		if (la.kind == 6) {
			Get();
			Ident(out id);
			Expect(7);
			Boolit(out b);
			gen.printVariableBool(id, b); 
		} else if (la.kind == 8) {
			Get();
			Ident(out id);
			Expect(7);
			Stringlit(out s);
			gen.printVariableString(id, s); 
		} else if (la.kind == 9) {
			Get();
			Ident(out id);
			Expect(7);
			Intlit(out i);
			gen.printVariableInt(id, i); 
		} else SynErr(40);
	}

	void Ident(out string id) {
		Expect(1);
		id = t.val; 
	}

	void Boolit(out string b) {
		b = t.val; 
		if (la.kind == 21) {
			Get();
			b = t.val; 
		} else if (la.kind == 22) {
			Get();
			b = t.val; 
		} else SynErr(41);
	}

	void Stringlit(out string s) {
		Expect(3);
		s = t.val; 
	}

	void Intlit(out string i) {
		Intlitsign();
		i = t.val; 
	}

	void Statement() {
		switch (la.kind) {
		case 10: {
			Compstat();
			break;
		}
		case 1: {
			Assign();
			break;
		}
		case 12: {
			Input();
			break;
		}
		case 14: {
			Output();
			break;
		}
		case 16: {
			Ifstat();
			break;
		}
		case 20: {
			Whilestat();
			break;
		}
		default: SynErr(42); break;
		}
	}

	void Assign() {
		string id = "";
		string express = "";
		
		Ident(out id);
		Expect(7);
		Express();
		Expect(4);
		express = t.val; // this gets ";"
		gen.printAssign(id, express);
		
	}

	void Input() {
		string id = ""; 
		Expect(12);
		Expect(13);
		Ident(out id);
		Expect(4);
	}

	void Output() {
		Expect(14);
		Expect(15);
		Express();
		Expect(4);
	}

	void Ifstat() {
		Expect(16);
		Expect(17);
		Express();
		Expect(18);
		Statement();
		if (la.kind == 19) {
			Get();
			Statement();
		}
	}

	void Whilestat() {
		Expect(20);
		Expect(17);
		Express();
		Expect(18);
		Statement();
	}

	void Statements() {
		while (StartOf(2)) {
			Statement();
		}
	}

	void Express() {
		Basic();
		if (StartOf(3)) {
			Relop();
			Express();
		}
	}

	void Literal() {
		if (la.kind == 21 || la.kind == 22) {
			string b = ""; 
			Boolit(out b);
		} else if (la.kind == 3) {
			string s = ""; 
			Stringlit(out s);
		} else if (la.kind == 2 || la.kind == 23 || la.kind == 24) {
			string i = ""; 
			Intlit(out i);
		} else SynErr(43);
	}

	void Sign() {
		if (la.kind == 23) {
			Get();
		} else if (la.kind == 24) {
			Get();
		} else SynErr(44);
	}

	void Basic() {
		Term();
		if (la.kind == 23 || la.kind == 24 || la.kind == 30) {
			Addop();
			Basic();
		}
	}

	void Relop() {
		switch (la.kind) {
		case 31: {
			Get();
			break;
		}
		case 32: {
			Get();
			break;
		}
		case 33: {
			Get();
			break;
		}
		case 34: {
			Get();
			break;
		}
		case 35: {
			Get();
			break;
		}
		case 36: {
			Get();
			break;
		}
		default: SynErr(45); break;
		}
	}

	void Term() {
		Factor();
		if (StartOf(4)) {
			Mulop();
			Term();
		}
	}

	void Addop() {
		if (la.kind == 23) {
			Get();
		} else if (la.kind == 24) {
			Get();
		} else if (la.kind == 30) {
			Get();
		} else SynErr(46);
	}

	void Factor() {
		string id = ""; 
		if (StartOf(5)) {
			Literal();
		} else if (la.kind == 1) {
			Ident(out id);
		} else if (la.kind == 17) {
			Get();
			Express();
			Expect(18);
		} else if (la.kind == 25) {
			Get();
			Factor();
		} else SynErr(47);
	}

	void Mulop() {
		if (la.kind == 26) {
			Get();
		} else if (la.kind == 27) {
			Get();
		} else if (la.kind == 28) {
			Get();
		} else if (la.kind == 29) {
			Get();
		} else SynErr(48);
	}

	void Intlitsign() {
		if (la.kind == 23 || la.kind == 24) {
			Sign();
		}
		Expect(2);
	}



	public void Parse() {
		la = new Token();
		la.val = "";		
		Get();
		MSB_CMM();

    Expect(0);
	}
	
	static readonly bool[,] set = {
		{T,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x},
		{x,x,x,x, x,T,T,x, T,T,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x},
		{x,T,x,x, x,x,x,x, x,x,T,x, T,x,T,x, T,x,x,x, T,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x},
		{x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,T, T,T,T,T, T,x,x},
		{x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,x,T,T, T,T,x,x, x,x,x,x, x,x,x},
		{x,x,T,T, x,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x, x,T,T,T, T,x,x,x, x,x,x,x, x,x,x,x, x,x,x}

	};
} // end Parser


public class Errors {
	public int count = 0;                                    // number of errors detected
	public System.IO.TextWriter errorStream = Console.Out;   // error messages go to this stream
  public string errMsgFormat = "-- line {0} col {1}: {2}"; // 0=line, 1=column, 2=text
  
	public void SynErr (int line, int col, int n) {
		string s;
		switch (n) {
			case 0: s = "EOF expected"; break;
			case 1: s = "ident expected"; break;
			case 2: s = "uint expected"; break;
			case 3: s = "stringlit expected"; break;
			case 4: s = "\";\" expected"; break;
			case 5: s = "\"const\" expected"; break;
			case 6: s = "\"bool\" expected"; break;
			case 7: s = "\"=\" expected"; break;
			case 8: s = "\"string\" expected"; break;
			case 9: s = "\"int\" expected"; break;
			case 10: s = "\"{\" expected"; break;
			case 11: s = "\"}\" expected"; break;
			case 12: s = "\"cin\" expected"; break;
			case 13: s = "\">>\" expected"; break;
			case 14: s = "\"cout\" expected"; break;
			case 15: s = "\"<<\" expected"; break;
			case 16: s = "\"if\" expected"; break;
			case 17: s = "\"(\" expected"; break;
			case 18: s = "\")\" expected"; break;
			case 19: s = "\"else\" expected"; break;
			case 20: s = "\"while\" expected"; break;
			case 21: s = "\"false\" expected"; break;
			case 22: s = "\"true\" expected"; break;
			case 23: s = "\"+\" expected"; break;
			case 24: s = "\"-\" expected"; break;
			case 25: s = "\"!\" expected"; break;
			case 26: s = "\"*\" expected"; break;
			case 27: s = "\"/\" expected"; break;
			case 28: s = "\"%\" expected"; break;
			case 29: s = "\"&&\" expected"; break;
			case 30: s = "\"||\" expected"; break;
			case 31: s = "\"<\" expected"; break;
			case 32: s = "\"<=\" expected"; break;
			case 33: s = "\"==\" expected"; break;
			case 34: s = "\">=\" expected"; break;
			case 35: s = "\">\" expected"; break;
			case 36: s = "\"!=\" expected"; break;
			case 37: s = "??? expected"; break;
			case 38: s = "invalid Declaration"; break;
			case 39: s = "invalid ConstantDec"; break;
			case 40: s = "invalid VariableDec"; break;
			case 41: s = "invalid Boolit"; break;
			case 42: s = "invalid Statement"; break;
			case 43: s = "invalid Literal"; break;
			case 44: s = "invalid Sign"; break;
			case 45: s = "invalid Relop"; break;
			case 46: s = "invalid Addop"; break;
			case 47: s = "invalid Factor"; break;
			case 48: s = "invalid Mulop"; break;

			default: s = "error " + n; break;
		}
		errorStream.WriteLine(errMsgFormat, line, col, s);
		count++;
	}

	public void SemErr (int line, int col, string s) {
		errorStream.WriteLine(errMsgFormat, line, col, s);
		count++;
	}
	
	public void SemErr (string s) {
		errorStream.WriteLine(s);
		count++;
	}
	
	public void Warning (int line, int col, string s) {
		errorStream.WriteLine(errMsgFormat, line, col, s);
	}
	
	public void Warning(string s) {
		errorStream.WriteLine(s);
	}
} // Errors


public class FatalError: Exception {
	public FatalError(string m): base(m) {}
}

