﻿//Copyright (c) 2009, Mark S. Burrell
//All rights reserved.

//Redistribution and use in source and binary forms, with or without modification, are 
//permitted provided that the following conditions are met:

//    * Redistributions of source code must retain the above copyright notice, this list 
//of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this 
//list of conditions and the following disclaimer in the documentation and/or other 
//materials provided with the distribution.

//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
//EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
//OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
//SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
//OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
//SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using Sedici.Hardware;


namespace Sedici.FirmwareBuilder
{
    public partial class FirmwareBuilder : Form
    {
        // positions in the opcode arrays
        private const ushort FETCH = 256;
        private const ushort INTERRUPT = 257;

        private formFirmware dlgFirmware = new formFirmware();

        // define Opcodes object
        public Firmware firmware = new Firmware();

        int currentOpcode = 0;

        string currentFirmwareFile = null;

        bool updated = false;

        public FirmwareBuilder()
        {
            InitializeComponent();

            for (ushort i = 0; i <= INTERRUPT; i++)
            {
                if (i < FETCH)
                {
                    string istring = i.ToString("X").PadLeft(2, '0');
                    opcodesList.Items.Add(istring);
                }
                else if (i == FETCH)
                {
                    opcodesList.Items.Add("FETCH");
                }
                else
                {
                    opcodesList.Items.Add("INTERRUPT");
                }

            }

            updateFirmwareList();

        }

        private void fileExit(object sender, EventArgs e)
        {


            DialogResult quit = MessageBox.Show("Do you want to exit?","Exit?", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

            if (quit == DialogResult.Yes)
            {
                if (updated)
                {
                    DialogResult reply = MessageBox.Show("You made changes.  Do you want to save them before I close down?",
                        "Save firmware?", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                    if (reply == DialogResult.Yes)
                    {
                        fileSaveAs(sender, e);
                    }

                }
                Application.Exit();
            }

        }

        private void fileRead(object sender, EventArgs e)
        {

            if (updated)
            {
                DialogResult reply = MessageBox.Show("You have made unsaved changes.  Do you want me to continue loading a new file?",
                    "Really open a new file?", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                if (reply == DialogResult.No)
                {
                    return;
                }
            }

            // read the file into a data structure
            OpenFileDialog dlgOpen = new OpenFileDialog();

            dlgOpen.Filter = "Firmware Files (*.frm)|*.frm|All Files (*.*)|*.*";
            dlgOpen.Title = "Open Firmware File ...";

            if (dlgOpen.ShowDialog() == DialogResult.OK)
            {
                // open the file and load
                //Console.Write(dlgOpen.FileName);

                //clear the firmware
                firmware.clearAllOpcodes();

                //clear the itemlist
                opcodesList.Items.Clear();
                mnemonicTextBox.Clear();
                firmwareTextBox.Clear();
                firmwareDescTextBox.Clear();
                //opcodeTextBox.Clear();
                descriptionTextBox.Clear();

                currentFirmwareFile = dlgOpen.FileName;

                if (checkBoxSaveFormat.Checked)
                {
                    // load nicer format
                    firmware.LoadFirmware(dlgOpen.FileName);
                }
                else
                {
                    firmware.LoadFirmwareOldFormat(dlgOpen.FileName);
                }


                for (ushort i = 0; i <= INTERRUPT; i++ )
                {
                    if (i < FETCH)
                    {
                        //string istring = supportMethods.FormatUshort((ushort)i, Base.hex);
                        string istring = i.ToString("X").PadLeft(2, '0');
                        string fred = firmware.getMnemonic(i);
                        opcodesList.Items.Add(istring + "  " + fred);
                    }
                    else if (i == FETCH)
                    {
                        opcodesList.Items.Add("FETCH");
                    }
                    else
                    {
                        opcodesList.Items.Add("INTERRUPT");
                    }

                }

                firmwareDescTextBox.Text = firmware.Comment;

                // set the details for opcode 0
                int opcode = 0;
                currentOpcode = opcode;

                //opcodeTextBox.Text = opcode.ToString("X").PadLeft(2, '0');

                mnemonicTextBox.Text = firmware.getMnemonic((ushort)opcode);

                descriptionTextBox.Text = firmware.getDescription((ushort)opcode);

                ArrayList fred2 = firmware.getRawMicroInstructions((ushort)opcode);

                string firmwarestuff = "";

                for (int i = 0; i < fred2.Count; i++)
                {
                    firmwarestuff = firmwarestuff + fred2[i].ToString() + Environment.NewLine;
                }

                firmwareTextBox.Text = firmwarestuff;

                //labelOpcode.Text = "Opcode : 00";

                buttonUpdate.Enabled = false;

                updated = false;


            }
        }

        private void fileSave(object sender, EventArgs e)
        {
            // do a save rather than a save as

            // If the file name is not an empty string open it for saving.
            if (currentFirmwareFile != null)
            {
                if (checkBoxSaveFormat.Checked)
                {
                    // Save in old format
                    performSave(currentFirmwareFile);
                }
                else
                {
                    performSaveOldFormat(currentFirmwareFile);
                }
            }
            else
            {
                fileSaveAs(sender, e);
            }

        }

        private void fileSaveAs(object sender, EventArgs e)
        {
            SaveFileDialog dlgSave = new SaveFileDialog();
            dlgSave.Filter = "Firmware File|*.frm";
            dlgSave.Title = "Save a Firmware File ...";
            dlgSave.ShowDialog();

            if (dlgSave.FileName != "")
            {
                if (checkBoxSaveFormat.Checked)
                {
                    // Save in old format
                    performSave(dlgSave.FileName);
                }
                else
                {
                    performSaveOldFormat(dlgSave.FileName);
                }

            }



        }

        private void updateOpcodesList(object sender, EventArgs e)
        {
            int opcode = 0;

            // do we have the right line?
            if (opcodesList.SelectedIndex != -1)
            {
                opcode = opcodesList.Items.IndexOf(opcodesList.SelectedItem);
                currentOpcode = opcode;

                //if (opcode < FETCH)
                //{
                //    //opcodeTextBox.Text = opcode.ToString("X").PadLeft(2, '0');
                //}
                //else if (opcode == FETCH)
                //{
                //    //opcodeTextBox.Text = "FETCH";
                //}
                //else
                //{
                //    //opcodeTextBox.Text = "INTERRUPT";
                //}

                //labelOpcode.Text = "Opcode : " + opcode.ToString("X").PadLeft(2, '0');

                mnemonicTextBox.Text = firmware.getMnemonic((ushort)opcode);

                descriptionTextBox.Text = firmware.getDescription((ushort)opcode);

                ArrayList fred = firmware.getRawMicroInstructions((ushort)opcode);

                string firmwarestuff = "";
                firmwareTextBox.Clear();
                for (int i = 0; i < fred.Count; i++)
                {
                    firmwarestuff = firmwarestuff + fred[i].ToString() + Environment.NewLine;
                }

                firmwareTextBox.Text = firmwarestuff;

                buttonUpdate.Enabled = false;
            }

        }

        private void firmwareDescTextBox_TextChanged(object sender, EventArgs e)
        {
            buttonUpdate.Enabled = true;
        }

        private void mnemonicTextBox_TextChanged(object sender, EventArgs e)
        {
            buttonUpdate.Enabled = true;
        }

        private void descriptionTextBox_TextChanged(object sender, EventArgs e)
        {
            buttonUpdate.Enabled = true;
        }

        private void firmwareTextBox_TextChanged(object sender, EventArgs e)
        {
            buttonUpdate.Enabled = true;
        }

        private void updateOpcodeFirmware(object sender, EventArgs e)
        {
            updated = true; // make sure close dialog asks if user wants to save

            int currentSelectedIndex = opcodesList.SelectedIndex;

            string[] spaceseparator = { " " };
            string[] separator = { Environment.NewLine };
            string[] lines = { };
            string[] lineElements = { };
            lines = firmwareTextBox.Text.Split(separator, StringSplitOptions.None);

            // clear all opcode details
            firmware.clearOpcode((ushort)currentOpcode);

            // now update
            for (int x = 0; x < lines.Length; x++)
            {
                firmware.setSingleRawMicroInstruction((ushort)currentOpcode, lines[x]);
            }

            firmware.clearComment();
            firmware.Comment = firmwareDescTextBox.Text;
            firmware.setDescription((ushort)currentOpcode, descriptionTextBox.Text);
            firmware.setMnemonic((ushort)currentOpcode, mnemonicTextBox.Text);

            // update the left hand column of opcodes
            opcodesList.Items.Clear();
            for (ushort i = 0; i <= INTERRUPT; i++)
            {
                if (i < FETCH)
                {
                    //string istring = supportMethods.FormatUshort((ushort)i, Base.hex);
                    string istring = i.ToString("X").PadLeft(2, '0');
                    string fred = firmware.getMnemonic(i);
                    opcodesList.Items.Add(istring + "  " + fred);
                }
                else if (i == FETCH)
                {
                    opcodesList.Items.Add("FETCH");
                }
                else
                {
                    opcodesList.Items.Add("INTERRUPT");
                }

            }

            opcodesList.SelectedIndex = currentSelectedIndex;

            buttonUpdate.Enabled = false;

            // now let's try to validate the code entered and append something like
            // <--- ERROR to any code that is invalid


            string newtext = "";
            string newline = "";
            for (int x = 0; x < lines.Length; x++)
            {
                // extract microcode first
                lineElements = lines[x].Split(spaceseparator, 2, StringSplitOptions.RemoveEmptyEntries);
                if (lineElements.Length > 0)
                {
                    if (lineElements[0].Substring(0, 1) == "*")
                    {
                        // the whole line is a comment
                        newline = lines[x];
                    }
                    else
                    {
                        string legality = firmware.validMicroInstruction(lineElements[0]);
                        if (legality == "illegal")
                        {
                            //MessageBox.Show(legality);

                            newline = lineElements[0] + " <-----|ERROR|";

                            if (lineElements.Length > 1)
                            {
                                newline = newline + lineElements[1];
                            }

                        }
                        else
                        {
                            newline = lines[x];
                        }
                    }



                    if (newtext == "")
                    {
                        newtext = newline;
                    }
                    else
                    {
                        newtext = newtext + Environment.NewLine + newline;
                    }




                }
                else
                {
                    if (newtext == "")
                    {
                        newtext = lines[x];
                    }
                    else
                    {
                        newtext = newtext + Environment.NewLine + lines[x];
                    }
                }

            }

            firmwareTextBox.Text = newtext;


        }

        private void updateFirmwareList()
        {

            string s =

                    "" + "----------------------------" + Environment.NewLine +
                    "" + "Flags" + Environment.NewLine +
                    "" + "----------------------------" + Environment.NewLine +
                    "if(psr(c)==1)" + Environment.NewLine +
                    "if(psr(c)==0)" + Environment.NewLine +
                    "if(psr(n)==1)" + Environment.NewLine +
                    "if(psr(n)==0)" + Environment.NewLine +
                    "if(psr(z)==1)" + Environment.NewLine +
                    "if(psr(z)==0)" + Environment.NewLine +
                    "if(psr(v)==1)" + Environment.NewLine +
                    "if(psr(v)==0)" + Environment.NewLine +

                    // ALU microinstructions
                    "" + Environment.NewLine +
                    "" + "----------------------------" + Environment.NewLine +
                    "" + "ALU instructions" + Environment.NewLine +
                    "" + "----------------------------" + Environment.NewLine +
                    "alur=[alux]+[aluy]" + Environment.NewLine +
                    "alur=[alux]+[aluy]+[psr(c)]" + Environment.NewLine +
                    "alur=[alux]-[aluy]" + Environment.NewLine +
                    "alur=[alux]<<1" + Environment.NewLine +
                    "alur=[alux]>>1" + Environment.NewLine +
                    "alur=[alux]&[aluy]" + Environment.NewLine +
                    "alur=[alux]|[aluy]" + Environment.NewLine +
                    "alur=~[alux]" + Environment.NewLine +
                    "alur=[alux]+1" + Environment.NewLine +
                    "alur=[alux]-1" + Environment.NewLine +

                    // general control
                    "" + Environment.NewLine +
                    "" + "----------------------------" + Environment.NewLine +
                    "" + "General control instructions" + Environment.NewLine +
                    "" + "----------------------------" + Environment.NewLine +
                    "nop" + Environment.NewLine +
                    "halt" + Environment.NewLine +

                    // interrupt flags
                    "" + Environment.NewLine +
                    "" + "----------------------------" + Environment.NewLine +
                    "" + "Interrupt flags" + Environment.NewLine +
                    "" + "----------------------------" + Environment.NewLine +
                    "psr(i)=1" + Environment.NewLine +
                    "psr(i)=0" + Environment.NewLine +
                    "psr(e)=1" + Environment.NewLine +
                    "psr(e)=0" + Environment.NewLine +

                    // supervisor flag
                    "" + Environment.NewLine +
                    "" + "----------------------------" + Environment.NewLine +
                    "" + "Supervisor flag" + Environment.NewLine +
                    "" + "----------------------------" + Environment.NewLine +
                    "psr(s)=1" + Environment.NewLine +
                    "psr(s)=0" + Environment.NewLine +

                    // specific data movements
                    "" + Environment.NewLine +
                    "" + "----------------------------" + Environment.NewLine +
                    "" + "Specific data movements" + Environment.NewLine +
                    "" + "----------------------------" + Environment.NewLine +
                    "cu<-[ir(opcode)]" + Environment.NewLine +
                    "m[mar]<-[mdr]" + Environment.NewLine +
                    "mdr<-[m[mar]]" + Environment.NewLine +
                    "pc<-[inc]" + Environment.NewLine +
                    "inc<-[pc]" + Environment.NewLine +

                    // general data movements
                    "" + Environment.NewLine +
                    "" + "----------------------------" + Environment.NewLine +
                    "" + "General data movements" + Environment.NewLine +
                    "" + "in form dest<-[src] where" + Environment.NewLine +
                    "" + "src/dest can be from:" + Environment.NewLine +
                    "" + "----------------------------" + Environment.NewLine +
                    "d0-d7" + Environment.NewLine +
                    "mar" + Environment.NewLine +
                    "mdr" + Environment.NewLine +
                    "alux" + Environment.NewLine +
                    "aluy" + Environment.NewLine +
                    "alur" + Environment.NewLine +
                    "usp" + Environment.NewLine +
                    "ssp" + Environment.NewLine +
                    "pc" + Environment.NewLine +
                    "psr" + Environment.NewLine +
                    "ir" + Environment.NewLine +
                    "ir(r0)" + Environment.NewLine +
                    "ir(r1)" + Environment.NewLine +
                    "ir(operand)" + Environment.NewLine +
                    "psr(intvec)" + Environment.NewLine +
                    "jumpers(intbase) (src only)" + Environment.NewLine +
                    "" + "----------------------------" + Environment.NewLine +
                    "" + "r0 and r1 indices" + Environment.NewLine +
                    "" + "----------------------------" + Environment.NewLine +
                    "0-7 d0-d7" + Environment.NewLine +
                    "8   usp" + Environment.NewLine +
                    "9   ssp" + Environment.NewLine +
                    "10-15 undefined";



            dlgFirmware.addText(s);

        }

        private void showFirmwareList(object sender, EventArgs e)
        {
            // open the firmware list dialog
            dlgFirmware.Show();

        }

        private void performSave(string filename)
        {
            //MessageBox.Show("Looking to save [" + filename + "]");

            string[] lines = { };
            string[] separator = { Environment.NewLine };

            if (filename != null)
            {
                StreamWriter sw;
                string[] details = { };

                try
                {
                    sw = File.CreateText(filename);
                }
                catch (IOException)
                {
                    MessageBox.Show("File creation failed for some reason, malformed filename?");
                    return;
                }

                // save comment first
                lines = firmware.Comment.Split(separator, StringSplitOptions.None);
                for (int x = 0; x < lines.Length; x++)
                {
                    sw.WriteLine(lines[x]);
                }

                sw.WriteLine();
                //sw.WriteLine();

                // now we write  each opcode in turn

                for (int opcode = 0; opcode <= INTERRUPT; opcode++)
                {
                    if (firmware.getMnemonic((ushort)opcode) == "")
                    {

                    }
                    else
                    {

                        //sw.Write("#I!");
                        sw.Write("*---------");
                        sw.Write("----------");
                        sw.Write("----------");
                        sw.Write("----------");
                        sw.Write("----------");
                        sw.Write("----------");
                        sw.Write("----------");
                        sw.Write("----------");
                        sw.WriteLine();
                        if (opcode == FETCH)
                        {
                            sw.WriteLine("Opcode        fetch");
                        }
                        else if (opcode == INTERRUPT)
                        {
                            sw.WriteLine("Opcode        interrupt");
                        }
                        else
                        {
                            sw.Write("Opcode        ");
                            sw.WriteLine(opcode.ToString("X").PadLeft(2, '0').ToUpper());
                        }
                        sw.WriteLine("Mnemonic      \"" + firmware.getMnemonic((ushort)opcode) + "\"");
                        sw.WriteLine("Description   \"" + firmware.getDescription((ushort)opcode) + "\"");
                        sw.WriteLine();

                        // now write the microinstruction list
                        foreach (string m in firmware.getRawMicroInstructions((ushort)opcode))
                        {
                            if (m == "")
                            {
                                // ignore blank lines
                            }
                            else
                            {
                                sw.WriteLine(m);
                            }
                        }
                        sw.WriteLine();
                    }


                }


                sw.Close();

                updated = false;
            }

        }

        private void performSaveOldFormat(string filename)
        {
            //MessageBox.Show("Looking to save [" + filename + "]");

            string[] lines = { };
            string[] separator = { Environment.NewLine };

            if (filename != null)
            {
                StreamWriter sw;
                string[] details = { };

                try
                {
                    sw = File.CreateText(filename);
                }
                catch (IOException)
                {
                    MessageBox.Show("File creation failed for some reason, malformed filename?");
                    return;
                }

                // save comment first
                lines = firmware.Comment.Split(separator, StringSplitOptions.None);
                for (int x = 0; x < lines.Length; x++)
                {
                    sw.WriteLine("#C!" + lines[x]);
                }

                sw.WriteLine();
                //sw.WriteLine();

                // now we write  each opcode in turn

                for (int opcode = 0; opcode <= INTERRUPT; opcode++)
                {
                    if (firmware.getMnemonic((ushort)opcode) == "")
                    {

                    }
                    else
                    {

                        sw.Write("#I!");
                        if (opcode == FETCH)
                        {
                            sw.Write("FETCH");
                        }
                        else if (opcode == INTERRUPT)
                        {
                            sw.Write("INTERRUPT");
                        }
                        else
                        {
                            sw.Write(opcode.ToString("X").PadLeft(2, '0').ToUpper());
                        }
                        sw.Write("!" + firmware.getMnemonic((ushort)opcode));
                        sw.Write("!" + firmware.getDescription((ushort)opcode));
                        sw.WriteLine();

                        // now write the microinstruction list
                        foreach (string m in firmware.getRawMicroInstructions((ushort)opcode))
                        {
                            if (m == "")
                            {
                            }
                            else
                            {
                                sw.WriteLine("#F!" + m);
                            }
                        }
                        sw.WriteLine();
                    }


                }


                sw.Close();

                updated = false;
            }

        }

        private void showAboutBox(object sender, EventArgs e)
        {
            formAbout about = new formAbout();
            about.ShowDialog();
        }  
 
    }
}
