//Copyright (c) 2009, Mark S. Burrell
//All rights reserved.

//Redistribution and use in source and binary forms, with or without modification, are 
//permitted provided that the following conditions are met:

//    * Redistributions of source code must retain the above copyright notice, this list 
//of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this 
//list of conditions and the following disclaimer in the documentation and/or other 
//materials provided with the distribution.

//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
//EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
//OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
//SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
//OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
//SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

using System;
using System.Collections.Generic;

namespace Sedici.Hardware
{
    public abstract class Chip
    {
        private ushort baseAddress;
        private bool installed = true;

        public Chip(ushort address)
        {
            baseAddress = address;
            HardReset();
            return;
        }

        public Chip()
        {
            baseAddress = 0;
            HardReset();
            return;
        }

        public bool Installed
        {
            get
            {
                return (installed);
            }
            set
            {
                installed = value;
            }
        }

        public ushort BaseAddress
        {
            get
            {
                return (baseAddress);
            }
            set
            {
                baseAddress = value;
            }
        }

        public bool Contains(ushort address)
        {
            bool contains = false;

            if ((Installed) &&(address >= BaseAddress) && (address < (BaseAddress + Size)))
            {
                contains = true;
            }
            return (contains);
        }

        public abstract ushort Size
        {
            get;
        }

        public abstract ushort Read(ushort address);

        public abstract void Write(ushort address, ushort value);

        public abstract void SoftReset();

        public abstract void HardReset();

    }       
}