//Copyright (c) 2009, Mark S. Burrell
//All rights reserved.

//Redistribution and use in source and binary forms, with or without modification, are 
//permitted provided that the following conditions are met:

//    * Redistributions of source code must retain the above copyright notice, this list 
//of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this 
//list of conditions and the following disclaimer in the documentation and/or other 
//materials provided with the distribution.

//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
//EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
//OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
//SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
//OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
//SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

using System;
using System.Collections.Generic;

namespace Sedici.Hardware
{
    public class Chip_RAM64K : Chip
    {
        // 32K addresses of 16-bit words

        private const ushort SIZE = 0x8000;

        // actual storage for memory
        private ushort[] memory = new ushort[SIZE];

        public Chip_RAM64K(ushort value)
        {
            BaseAddress = value;
            HardReset();
            return;
        }

        override public ushort Size
        {
            get
            {
                return (SIZE);
            }
        }

        override public ushort Read(ushort address)
        {
            ushort value = 0;
            if ((address >= BaseAddress) && (address < (BaseAddress + SIZE)))
            {
                address = (ushort)(address - BaseAddress);
                value = memory[address];
            }
            return (value);
        }

        override public void Write(ushort address, ushort value)
        {
            if ((address >= BaseAddress) && (address < (BaseAddress + SIZE)))
            {
                address = (ushort)(address - BaseAddress);
                memory[address] = value;
            }
            return;
        }

        override public void HardReset()
        {

            // Initialise memory
            for (int i = 0; i < SIZE; i++)
            {
                memory[i] = 0x0000;
            }
            return;
        }

        override public void SoftReset()
        {
            // does nothing
            return;
        }

    }


}