//Copyright (c) 2009, Mark S. Burrell
//All rights reserved.

//Redistribution and use in source and binary forms, with or without modification, are 
//permitted provided that the following conditions are met:

//    * Redistributions of source code must retain the above copyright notice, this list 
//of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this 
//list of conditions and the following disclaimer in the documentation and/or other 
//materials provided with the distribution.

//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
//EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
//OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
//SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
//OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
//SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;

namespace Sedici.Hardware
{

    public class Memory
    {
        // instantiate memory chips
        Chip_RAM64K           memoryChip1  = new Chip_RAM64K(0x0000);
        Chip_RAM64K           memoryChip2  = new Chip_RAM64K(0x8000);

        public Chip_ROM       romChip      = new Chip_ROM(0x0000);
        public Chip_IO        ioChip       = new Chip_IO(0x0010);
        public Chip_Clock     clockChip    = new Chip_Clock(0x0020);
        public Chip_Graphics  graphicsChip = new Chip_Graphics(0x6000);

        public Memory()
        {
            HardReset();
            return;
        }

        public void HardReset()
        {
            ioChip.HardReset();
            graphicsChip.HardReset();
            memoryChip1.HardReset();
            memoryChip2.HardReset();
            romChip.HardReset();
            clockChip.HardReset();
            return;
        }

        public void SoftReset()
        {
            ioChip.SoftReset();
            graphicsChip.SoftReset();
            memoryChip1.SoftReset();
            memoryChip2.SoftReset();
            romChip.SoftReset();
            clockChip.SoftReset();
            return;
        }

        public ushort Read(ushort address)
        {
            ushort value = 0;

            if (clockChip.Contains(address))         value = clockChip.Read(address);
            else if (ioChip.Contains(address))       value = ioChip.Read(address);
            else if (graphicsChip.Contains(address)) value = graphicsChip.Read(address);
            else if (romChip.Contains(address))      value = romChip.Read(address);
            else if (memoryChip1.Contains(address))  value = memoryChip1.Read(address);
            else if (memoryChip2.Contains(address))  value = memoryChip2.Read(address);
            return (value);
        }

        public void Write(ushort address, ushort value)
        {
                 if (clockChip.Contains(address))    clockChip.Write(address, value);
            else if (ioChip.Contains(address))       ioChip.Write(address, value);
            else if (graphicsChip.Contains(address)) graphicsChip.Write(address, value);
            else if (romChip.Contains(address))      { } // deliberate empty statement - do nothing
            else if (memoryChip1.Contains(address))  memoryChip1.Write(address, value);
            else if (memoryChip2.Contains(address))  memoryChip2.Write(address, value);
            return;
        }


        // this needs re-thought through in order to be removed.
        public string OutputBuffer()
        {
            return (ioChip.OutputBuffer());
        }

    }

}