//Copyright (c) 2009, Mark S. Burrell
//All rights reserved.

//Redistribution and use in source and binary forms, with or without modification, are 
//permitted provided that the following conditions are met:

//    * Redistributions of source code must retain the above copyright notice, this list 
//of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this 
//list of conditions and the following disclaimer in the documentation and/or other 
//materials provided with the distribution.

//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
//EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
//OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
//SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
//OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
//SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;

namespace Sedici.Hardware
{
    enum messageType { Information, Warning, Error, Fatal }

    class Message
    {
        public messageType Type;
        public string Content;

        public Message(messageType m, string message)
        {
            Type = m;
            Content = message;
        }
    }


    public class Messages
    {
        
        private ArrayList list = new ArrayList(); // will contain Message objects

        public Messages()
        {
            return;
        }

        public int Count
        {
            get {
                return(list.Count);
            }
        }

        public void AddWarning(string s)
        {
            Add(messageType.Warning, s);
            return;
        }

        public void AddError(string s)
        {
            Add(messageType.Error, s);
            return;
        }

        public void AddFatal(string s)
        {
            Add(messageType.Fatal, s);
            return;
        }

        public void AddInformational(string s)
        {
            Add(messageType.Information, s);
            return;
        }

        public ArrayList Retrieve()
        {
            // deliver all messages as an ArrayList of strings then wipe the list
            ArrayList a = new ArrayList();

            Message message;
            string s = "";

            for (int i = 0; i < list.Count; i++ )
            {
                message = (Message)list[i];
                if (message.Type == messageType.Information)
                {
                    s = "Info    - ";
                }
                else if (message.Type == messageType.Warning)
                {
                    s = "Warning - ";
                }
                else if (message.Type == messageType.Error)
                {
                    s = "Error   - ";
                }
                else if (message.Type == messageType.Fatal)
                {
                    s = "Fatal   - ";
                }
                s = s + message.Content;
                a.Add(s);

            }

            list.Clear();

            return (a);
        }

        private void Add(messageType m, string s)
        {
            // create Message object and add to the list
            Message message = new Message(m, s);
            list.Add(message);
        }

    }
}
