//Copyright (c) 2009, Mark S. Burrell
//All rights reserved.

//Redistribution and use in source and binary forms, with or without modification, are 
//permitted provided that the following conditions are met:

//    * Redistributions of source code must retain the above copyright notice, this list 
//of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this 
//list of conditions and the following disclaimer in the documentation and/or other 
//materials provided with the distribution.

//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
//EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
//OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
//SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
//OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
//SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

using System;
using System.IO;
using System.Collections.Generic;

namespace Sedici.Hardware
{
    public class supportMethods
    {
        public static string FormatUshort(ushort number, Base b)
        {
            string s = "";
            s = FormatUshort(number, b, 4);
            return (s);
        }

        public static string FormatUshort(ushort number, Base b, int padding)
        {
            ushort mask = 0x8000;
            string s = "";

            if (b == Base.hex)
            {
                ushort result = number;

                s = result.ToString("X").PadLeft(padding, '0').ToLower();
            }
            else if (b == Base.binary)
            {
                for (int i = 0; i < 16; i++)
                {
                    s = s + ((number & mask) == 0 ? "0" : "1");
                    mask = (ushort)(mask / 2);
                }
            }
            return (s);
        }

        public static int isNumber(string value, ref ushort result)
        {
            // return 1 = good number
            // return 0 = fail
            string leadingChar = " ";
            string nextChar = " ";
            int total = 0;
            int indchar = 0;
            ushort ushorttotal = 0;
            int startpos = 0;

            // do we have a vlaid string?
            if (value.Length == 0) {
                return (0);
            }

            // first check for a leading % ^ @ or $ if none the assume value is hex

            leadingChar = value.Substring(0, 1);

            if ((leadingChar == "^" || leadingChar == "%" || leadingChar == "@" || leadingChar == "$") && (value.Length < 2))
            {
                return (0);
            }

            switch (leadingChar)
            {
                case "^":
                    //Console.WriteLine("Decimal");
                    for (int i = 1; i < value.Length; i++)
                    {
                        nextChar = value.Substring(i, 1);
                        indchar = baseChar(nextChar, 10);
                        if (indchar == 255) { return (0); } else { total = (total * 10) + indchar; }
                    }
                    ushorttotal = (ushort)total;
                    if (ushorttotal == total) { result = ushorttotal; return (1); }
                    break;
                case "@":
                    //Console.WriteLine("Octal");
                    for (int i = 1; i < value.Length; i++)
                    {
                        nextChar = value.Substring(i, 1);
                        indchar = baseChar(nextChar, 8);
                        if (indchar == 255) { return (0); } else { total = (total * 8) + indchar; }
                    }
                    ushorttotal = (ushort)total;
                    if (ushorttotal == total) { result = ushorttotal; return (1); }
                    break;
                case "%":
                    //Console.WriteLine("Binary");
                    for (int i = 1; i < value.Length; i++)
                    {
                        nextChar = value.Substring(i, 1);
                        indchar = baseChar(nextChar, 2);
                        if (indchar == 255) { return (0); } else { total = (total * 2) + indchar; }
                    }
                    ushorttotal = (ushort)total;
                    if (ushorttotal == total) { result = ushorttotal; return (1); }
                    break;
                case "$":
                default:
                    //Console.WriteLine("Hex");
                    if (leadingChar == "$")
                    {
                        startpos = 1;
                    }
                    for (int i = startpos; i < value.Length; i++)
                    {
                        nextChar = value.Substring(i, 1);
                        indchar = baseChar(nextChar, 16);
                        if (indchar == 255) { return (0); } else { total = (total * 16) + indchar; }
                    }
                    ushorttotal = (ushort)total;
                    if (ushorttotal == total) { result = ushorttotal; return (1); }
                    break;

            }
            return (0);
        }

        static int baseChar(string c, int numberBase)
        {
            int value;

            switch (numberBase)
            {
                case 2:
                    switch (c)
                    {
                        case "0":
                            value = 0;
                            break;
                        case "1":
                            value = 1;
                            break;
                        default: value = 255;
                            break;
                    }
                    break;
                case 8:
                    switch (c)
                    {
                        case "0":
                            value = 0;
                            break;
                        case "1":
                            value = 1;
                            break;
                        case "2":
                            value = 2;
                            break;
                        case "3":
                            value = 3;
                            break;
                        case "4":
                            value = 4;
                            break;
                        case "5":
                            value = 5;
                            break;
                        case "6":
                            value = 6;
                            break;
                        case "7":
                            value = 7;
                            break;
                        case "8":
                            value = 8;
                            break;
                        default: value = 255;
                            break;
                    }
                    break;
                case 10:
                    switch (c)
                    {
                        case "0":
                            value = 0;
                            break;
                        case "1":
                            value = 1;
                            break;
                        case "2":
                            value = 2;
                            break;
                        case "3":
                            value = 3;
                            break;
                        case "4":
                            value = 4;
                            break;
                        case "5":
                            value = 5;
                            break;
                        case "6":
                            value = 6;
                            break;
                        case "7":
                            value = 7;
                            break;
                        case "8":
                            value = 8;
                            break;
                        case "9":
                            value = 9;
                            break;
                        default: value = 255;
                            break;
                    }
                    break;
                case 16:
                    switch (c)
                    {
                        case "0":
                            value = 0;
                            break;
                        case "1":
                            value = 1;
                            break;
                        case "2":
                            value = 2;
                            break;
                        case "3":
                            value = 3;
                            break;
                        case "4":
                            value = 4;
                            break;
                        case "5":
                            value = 5;
                            break;
                        case "6":
                            value = 6;
                            break;
                        case "7":
                            value = 7;
                            break;
                        case "8":
                            value = 8;
                            break;
                        case "9":
                            value = 9;
                            break;
                        case "a":
                        case "A":
                            value = 10;
                            break;
                        case "b":
                        case "B":
                            value = 11;
                            break;
                        case "c":
                        case "C":
                            value = 12;
                            break;
                        case "d":
                        case "D":
                            value = 13;
                            break;
                        case "e":
                        case "E":
                            value = 14;
                            break;
                        case "f":
                        case "F":
                            value = 15;
                            break;
                        default:
                            value = 255;
                            break;
                    }
                    break;
                default:
                    value = 255;
                    break;
            }
            return (value);
        }


        public static int updateFileLocation(ref string filename, string envName)
        {
            // return value:
            // 0 : file exists in current directory, filename unchanged
            // 1 : file exists in environment variable directory, filename updated to reflect correct location
            // 2 : file does not exist in either location, filename not updated
            //
            // The return value doesn't have to be checked.  Instead just ensure that any file open
            // operations return a fail message
            //

            //Console.Error.WriteLine("Looking for [" + filename + "]");

            int retval = 2;

            string env = "";
            string updatedfilename = "";

            if (File.Exists(filename))
            {
                retval = 0;
            }
            else {
                env = Environment.GetEnvironmentVariable(envName);
                if (env != "")
                {
                    if (Directory.Exists(env))
                    {
                        updatedfilename = env + filename;
                        if (File.Exists(updatedfilename))
                        {
                            filename = updatedfilename;
                            retval = 1;
                        }
                    }
                }
            }

            //if (retval == 0)
            //{
            //    Console.Error.WriteLine("Found it");
            //}
            //else if (retval == 1)
            //{
            //    Console.Error.WriteLine("Found it at [" + filename + "]");
            //}
            //else
            //{
            //    Console.Error.WriteLine("Didn't find it");
            //}


            return (retval);
        }


    }

}